#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Setup utility for Mimecast log collection """
import base64
import getpass
import hashlib
import hmac
import json
import os
import uuid
import datetime
import sys
import pprint
import _winreg
from os.path import dirname, abspath
import requests


def get_path(name):
    try:
        registry_key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, r'SOFTWARE\\LogRhythm\scsm', 0, (_winreg.KEY_WOW64_64KEY + _winreg.KEY_READ))
        value = _winreg.QueryValueEx(registry_key, "STATEPATH")[0]     
        return value
    except WindowsError:
        return None

print '************************************************************************'
print '****** Welcome to the Mimecast for LogRhythm setup utility v1.3.1 ******'
print '************************************************************************'

config = {}
print 'Please enter the full path to the folder where data should be stored'
data_dir = raw_input('Mimecast Data directory: ')
 
print 'Please enter the full path to the folder where MTA logs should be moved to once read by the System Monitor Agent'
archivepath = raw_input('Archive path: ')

print 'Please enter the full path to the folder where MTA logs are currently stored'
SourcePath = raw_input('MTA log path: ')

print 'Please enter the full path to the folder where the System Monitor is reading logs from'
ReadPath = raw_input('System Monitor read path: ')

print 'Please enter the Log Source Id'
MsgSourceID = raw_input('Log Source Id: ')

print 'Please enter the Number of files to read'
MaxReadFiles = raw_input('Maximum number of files to maintain within the System Monitor read path: ')

print 'Please enter the Application Id'
appId = raw_input('Application Id: ')


print 'Please enter the Application Key'
appKey = raw_input('Application Key: ')

path = get_path("STATEPATH")

posDir = os.path.join(path, "state", MsgSourceID)

config['data_dir'] = data_dir
# data_dir = os.path.join(dirname(dirname(abspath(__file__))), 'data')

print str(data_dir)

if os.path.exists(data_dir):
    print 'Data directory exists'
else:
    try:
        os.makedirs(data_dir)
        print 'Data directory created successfully'
        os.makedirs(os.path.join(data_dir, 'audit'))
        print 'Audit data directory created successfully'
        os.makedirs(os.path.join(data_dir, 'mta'))
        print 'MTA data directory created successfully'
        os.makedirs(os.path.join(data_dir, 'tmp'))
        print 'Tmp data directory created successfully'
    except Exception, e:
        print 'Unexpected error creating log directory: ' + str(e)

if os.path.exists(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'checkpoint')):
    print 'Checkpoint directory exists'
else:
    try:
        os.makedirs(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'checkpoint'))
        print 'Checkpoint directory created successfully'
    except Exception, e:
        print 'Unexpected error creating log directory: ' + str(e)


if os.path.exists(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'log')):
    print 'Log directory exists'
else:
    try:
        os.makedirs(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'log'))
        print 'Log directory created successfully'
    except Exception, e:
        print 'Unexpected error creating log directory: ' + str(e)



if os.path.exists(archivepath):
    print 'Archive directory exists'
else:
    try:
        os.makedirs(archivepath)
        print 'Archive directory created successfully'
    except Exception, e:
        print 'Unexpected error creating Archive directory: ' + str(e)


if os.path.exists(SourcePath):
    print 'Source directory exists'
else:
    try:
        os.makedirs(SourcePath)
        print 'Source directory created successfully'
    except Exception, e:
        print 'Unexpected error creating Source directory: ' + str(e)


if os.path.exists(ReadPath):
    print 'Read directory exists'
else:
    try:
        os.makedirs(ReadPath)
        print 'Read directory created successfully'
    except Exception, e:
        print 'Unexpected error creating Read directory: ' + str(e)



print 'Please log in using the email address and Mimecast password of a Mimecast administrator ' \
      'account to complete setup'



email_address = raw_input('Email Address:')
password = getpass.getpass(prompt='Password:')

print 'Setting base url for ' + email_address


def discover_base_url():
    try:
        base_url = "https://api.mimecast.com"
        uri = "/api/login/discover-authentication"
        url = base_url + uri

        request_id = str(uuid.uuid4())
        hdr_date = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S") + " UTC"

        headers = {
            'x-mc-app-id': appId,
            'x-mc-date': hdr_date,
            'x-mc-req-id': request_id,
            'Content-Type': 'application/json'
        }

        payload = {
            "data": [
                {
                    "emailAddress": email_address
                }
            ]
        }

        r = requests.post(url=url, headers=headers, data=str(payload))

        base_url = json.loads(r.text)
        return base_url['data'][0]['region']['api']
    except Exception, exception:
        print 'Unexpected error discovering base url, exception: ' + str(exception)
        sys.exit()


api_base_url = discover_base_url()
print 'Base URL discovered: ' + api_base_url
config['api_base_url'] = api_base_url
print 'Authenticating user to get access key and secret key values...'


def log_in():
    try:
        uri = "/api/login/login"
        url = api_base_url + uri
        request_id = str(uuid.uuid4())
        hdr_date = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S") + " UTC"

        headers = {
            'Authorization': 'Basic-Cloud ' + base64.b64encode(email_address + ':' + password),
            'x-mc-app-id': appId,
            'x-mc-date': hdr_date,
            'x-mc-req-id': request_id,
            'Content-Type': 'application/json'
        }

        payload = {
            "data": [
                {
                    "userName": email_address
                }
            ]
        }

        r = requests.post(url=url, headers=headers, data=str(payload))

        if r.status_code != 200:
            print 'Login Failed. Status Code: ' + str(r.status_code)
            print 'Response body: ' + r.text
            sys.exit()
        else:
            return r.text
    except Exception, exception:
        print 'Unexpected error processing log in, exception: ' + str(exception)
        sys.exit()

keys = json.loads(log_in())
access_key = keys['data'][0]['accessKey']
secret_key = keys['data'][0]['secretKey']
config['access_key'] = access_key
config['secret_key'] = secret_key
config['app_id'] = appId
config['app_key'] = appKey
config['archive_path'] = archivepath
config['posDir'] =  posDir
config['source_path'] =  SourcePath
config['read_path'] = ReadPath
config['msg_source_id'] = MsgSourceID
config['max_read_files'] = MaxReadFiles


print 'Log in successful. Getting account code and saving config...'


def get_account():
    uri = '/api/account/get-account'
    url = api_base_url + uri
    request_id = str(uuid.uuid4())
    hdr_date = datetime.datetime.utcnow().strftime("%a, %d %b %Y %H:%M:%S") + " UTC"

    hmac_sha1 = hmac.new(secret_key.decode("base64"), ':'.join([hdr_date, request_id, uri, appKey]),
                         digestmod=hashlib.sha1).digest()

    sig = base64.encodestring(hmac_sha1).rstrip()

    headers = {
        'Authorization': 'MC ' + access_key + ':' + sig,
        'x-mc-app-id': appId,
        'x-mc-date': hdr_date,
        'x-mc-req-id': request_id,
        'Content-Type': 'application/json'
    }

    payload = {
        'data': [
        ]
    }
    try:
        r = requests.post(url=url, headers=headers, data=str(payload))
    except Exception, exception:
        print 'Unexpected error getting account code for ' + email_address + '. Exception: ' + str(exception)
        sys.exit(0)

    return json.loads(r.text)['data'][0]['accountCode']


config['account_code'] = get_account()

try:
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'checkpoint', 'config.txt'), 'wb') as f:
        encoded_config = {k: unicode(v).encode("utf-8") for k, v in config.iteritems()}
        f.write(pprint.pformat(encoded_config))
except Exception, e:
    print 'Unexpected error writing config, exception: ' + str(e)
    sys.exit()

print 'Config saved successfully.'

