#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Script to fetch siem log data from the Mimecast API and add save to a folder for LogRhythm data collection"""

import base64
import hashlib
import hmac
import logging
import os
import uuid
import time
import datetime
import sys
import ast
import linecache
import shutil
from os.path import dirname, abspath
from zipfile import ZipFile
from itertools import islice



# Count number of files in directory

def number_of_files(folder):
    path, dirs, files = next(os.walk(folder))
    file_count = len(files)
 
    return int(file_count)


# Sort files by creation date (oldest to newest)

def sort_by_creation_date(directory):
    path = directory
    name_list = os.listdir(path)
    full_list = [os.path.join(path,i) for i in name_list]
    time_sorted_list = sorted(full_list, key=os.path.getmtime)
    return time_sorted_list


# Open config.txt and get archivepath, SourcePath, ReadPath, MsgSourceID and MaxReadFiles

with open(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                    'checkpoint', 'config.txt'), 'rb') as f:
    config = f.read()
    config = ast.literal_eval(config)
     


# Path to log file
log_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'log')

 #Path Agent will move files (with pos extensions) from AgentRead once they have been completely read.
archivepath = config['archive_path']

#Enter the "Log Source ID". This can be found by looking at the Log Source under the Log Sources tab in the console.
MsgSourceID = config['msg_source_id']


# Directory where pos files are located, path is extracted from Registry
# and "state" folder and message id is appended so full path to pos file is constructed
# for example C:\Program Files\LogRhythm\LogRhythm System Monitor\state\94

posDir = config['posDir']

#Path where source files (with siem extensions) are currently being downloaded or written to. 
# These files are created by api_setup.py script

SourcePath = config['source_path']   

#Path Agent will read the files from. This should be the same path as the "Flat File Path" in 
# the Log Source configuration, but different from the $SourcePath.

ReadPath = config['read_path']

 #Maximum number of files to have in the ReadPath folder at one time.
MaxReadFiles = int(config['max_read_files'])


# create log file

log_name = "MoveFiles-" + MsgSourceID + "_" + datetime.datetime.utcnow().strftime('%d%m%Y') + '.log'
logging.basicConfig(filename=os.path.join(log_dir, log_name), level=logging.INFO,
                    format='%(levelname)s|%(asctime)s|%(message)s')


#Starting Read:
logging.info("Executing Script.") 


#Checking if any files currently in read folder have been completely read, checking position file to verify all data has been ingested by Agent.

try:
    fileCount = 0
    logging.info('Verifying if any files have completed reading.')
    for posfile in os.listdir(posDir):
    
       filepath = os.path.join(posDir, posfile)
       
       file = linecache.getline(filepath, 1).strip()
       size = linecache.getline(filepath, 2).strip()
          
       if os.path.exists(file):
           
           if os.stat(file).st_size == int(size):
               if os.path.exists(file):
                   shutil.move(file, archivepath)
                   logging.info("File: " + file + " moved to: " + archivepath)
                   fileCount = fileCount + 1
                 
    logging.info(str(fileCount) + " files moved.")

except Exception, e:
    logging.error( "Error has occured " + str(e))

# Get number of files in ReadPath

FileCountReadPath = number_of_files(ReadPath)


# Get number of files in SourcePath

FileCountSourcePath = number_of_files(SourcePath)

# Checking if there is space in read folder, moving files from source to read by
# oldest creation time to read folder, staying under max files.

try:
    if FileCountReadPath < MaxReadFiles:
        logging.info("Attempting to move " + str(MaxReadFiles - FileCountReadPath) + " files to " + ReadPath)
        if FileCountSourcePath > 0:
            sorted_files = islice(sort_by_creation_date(SourcePath), int(MaxReadFiles - FileCountReadPath))
            for sorted_file in sorted_files:
                shutil.move(sorted_file, ReadPath)
                logging.info("Moved: " + sorted_file + " to: " + ReadPath)
  
        else:
            logging.info("No files to move from " + SourcePath + " to " + ReadPath + " - All files have been moved.")

    else:
        logging.info("Read folder full (Max = " + str(MaxReadFiles) + ") , not moving files")

    logging.info("Execution Complete")
except Exception, e:
    logging.error( "Error has occured " + str(e))    